@@ -8,6 +8,7 @@ import unittest
 import tempfile
 import re
 from unittest import mock
+from pathlib import Path
 
 from pkg_resources import parse_version
 
@@ -17,6 +18,14 @@ from render import render
 from git import from_vcs, from_keywords
 from subprocess_helper import run_command
 
+
+DEFAULT_PYPROJECT = """\
+[build-system]
+requires = ["setuptools"]
+build-backend = "setuptools.build_meta"
+"""
+
+
 class ParseGitDescribe(unittest.TestCase):
     def setUp(self):
         self.fakeroot = tempfile.mkdtemp()
@@ -361,7 +370,7 @@ VERBOSE = False
 
 class Repo(common.Common, unittest.TestCase):
 
-    # There are three tree states we're interested in:
+    # There are six tree states we're interested in:
     #  S1: sitting on the initial commit, no tags
     #  S2: dirty tree after the initial commit
     #  S3: sitting on the 1.0 tag
@@ -369,12 +378,13 @@ class Repo(common.Common, unittest.TestCase):
     #  S5: making a new commit after 1.0, clean tree
     #  S6: dirtying the tree after the post-1.0 commit
     #
-    # Then we're interested in 5 kinds of trees:
+    # Then we're interested in 6 kinds of trees:
     #  TA: source tree (with .git)
     #  TB: source tree without .git (should get 'unknown')
     #  TC: source tree without .git unpacked into prefixdir
     #  TD: git-archive tarball
-    #  TE: unpacked sdist tarball
+    #  TE: unpacked sdist tarball (python setup.py sdist)
+    #  TF: unpacked sdist tarball (python -m build -s)
     #
     # In three runtime situations:
     #  RA1: setup.py --version
@@ -389,7 +399,7 @@ class Repo(common.Common, unittest.TestCase):
     # or test/demoapp-script-only/)
 
     def test_full(self):
-        self.run_test("test/demoapp", False, ".")
+        self.run_test("test/demoapp", False, ".", False)
 
     def test_script_only(self):
         # This test looks at an application that consists entirely of a
@@ -398,17 +408,20 @@ class Repo(common.Common, unittest.TestCase):
         # anything executable. So of the 3 runtime situations examined by
         # Repo.test_full above, we only care about RB. (RA1 is valid too, but
         # covered by Repo).
-        self.run_test("test/demoapp-script-only", True, ".")
+        self.run_test("test/demoapp-script-only", True, ".", False)
 
     def test_project_in_subdir(self):
         # This test sets of the git repository so that the python project --
         # i.e. setup.py -- is not located in the root directory
-        self.run_test("test/demoapp", False, "project")
+        self.run_test("test/demoapp", False, "project", False)
 
     def test_no_tag_prefix(self):
-        self.run_test("test/demoapp", False, ".", tag_prefix="")
+        self.run_test("test/demoapp", False, ".", False, tag_prefix="")
 
-    def run_test(self, demoapp_dir, script_only, project_sub_dir, tag_prefix=None):
+    def test_pyproject(self):
+        self.run_test("test/demoapp-pyproject", False, ".", True)
+
+    def run_test(self, demoapp_dir, script_only, project_sub_dir, pep518, tag_prefix=None):
         # The test dir should live under /tmp/ or /var/ or somewhere that
         # isn't the child of the versioneer repo's .git directory, since that
         # will confuse the tests that check what happens when there is no
@@ -439,19 +452,40 @@ class Repo(common.Common, unittest.TestCase):
 
         shutil.copytree(demoapp_dir, self.projdir)
         setup_cfg_fn = self.project_file("setup.cfg")
-        with open(setup_cfg_fn, "r") as f:
-            setup_cfg = f.read()
-        setup_cfg = setup_cfg.replace("@VCS@", "git")
+        if os.path.exists(setup_cfg_fn):
+            with open(setup_cfg_fn, "r") as f:
+                setup_cfg = f.read()
+            setup_cfg = setup_cfg.replace("@VCS@", "git")
 
-        tag_prefix_regex = "tag_prefix = (.*)"
-        if tag_prefix is None:
-            tag_prefix = re.search(tag_prefix_regex, setup_cfg).group(1)
+            tag_prefix_regex = "tag_prefix = (.*)"
+            if tag_prefix is None:
+                tag_prefix = re.search(tag_prefix_regex, setup_cfg).group(1)
+            else:
+                setup_cfg = re.sub(tag_prefix_regex, f"tag_prefix = {tag_prefix}", setup_cfg)
+
+            with open(setup_cfg_fn, "w") as f:
+                f.write(setup_cfg)
+
+        if pep518:
+            # Set test versioneer build-system.requires entry to @ file:///<this-repo>
+            # so that `python -m build` picks it up.
+            pyproject_path = Path(self.project_file("pyproject.toml"))
+            versioneer_source_root = Path(__file__).absolute().parent.parent.parent
+            vsr = str(versioneer_source_root).replace("\\", "/")  # For testing on Windows...
+            pyproject_toml = pyproject_path.read_text()
+            pyproject_toml = pyproject_toml.replace("@REPOROOT@", f"file://{vsr}")
+
+            # Update versioneer config
+            pyproject_toml = pyproject_toml.replace("@VCS@", "git")
+            tag_prefix_regex = 'tag_prefix = "(.*)"'
+            if tag_prefix is None:
+                tag_prefix = re.search(tag_prefix_regex, pyproject_toml).group(1)
+            else:
+                pyproject_toml = re.sub(tag_prefix_regex, f'tag_prefix = "{tag_prefix}"', pyproject_toml)
+
+            pyproject_path.write_text(pyproject_toml)
         else:
-            setup_cfg = re.sub(tag_prefix_regex, f"tag_prefix = {tag_prefix}", setup_cfg)
-
-        with open(setup_cfg_fn, "w") as f:
-            f.write(setup_cfg)
-        shutil.copyfile("versioneer.py", self.project_file("versioneer.py"))
+            shutil.copyfile("versioneer.py", self.project_file("versioneer.py"))
         self.git("init")
         self.git("add", "--all")
         self.git("commit", "-m", "comment")
@@ -463,7 +497,10 @@ class Repo(common.Common, unittest.TestCase):
                         workdir=self.testdir)
         self.assertEqual(v, "0+untagged.1.g%s" % full[:7])
 
-        out = self.python("versioneer.py", "setup").splitlines()
+        if pep518:
+            out = self.python("-m", "versioneer", "install", "--no-vendor").splitlines()
+        else:
+            out = self.python("versioneer.py", "setup").splitlines()
         self.assertEqual(out[0], "creating src/demo/_version.py")
         init = os.path.join("src/demo", "__init__.py")
         if script_only:
@@ -496,7 +533,10 @@ class Repo(common.Common, unittest.TestCase):
         self.git("commit", "-m", "add _version stuff")
 
         # "versioneer.py setup" should be idempotent
-        out = self.python("versioneer.py", "setup").splitlines()
+        if pep518:
+            out = self.python("-m", "versioneer", "install", "--no-vendor").splitlines()
+        else:
+            out = self.python("versioneer.py", "setup").splitlines()
         self.assertEqual(out[0], "creating src/demo/_version.py")
         if script_only:
             self.assertEqual(out[1], f" {init} doesn't exist, ok")
@@ -516,6 +556,7 @@ class Repo(common.Common, unittest.TestCase):
                               "TC": [short, full, False, None],
                               "TD": ["0+unknown", full, False, NOTAG],
                               "TE": [short, full, False, None],
+                              "TF": [short, full, False, None],
                               })
 
         # TD: expanded keywords only tell us about tags and full revisionids,
@@ -534,6 +575,7 @@ class Repo(common.Common, unittest.TestCase):
                               "TC": [short, full, True, None],
                               "TD": ["0+unknown", full, False, NOTAG],
                               "TE": [short, full, True, None],
+                              "TF": [short, full, True, None],
                               })
 
         # S3: we commit that change, then make the first tag (1.0)
@@ -553,6 +595,7 @@ class Repo(common.Common, unittest.TestCase):
                               "TC": [short, full, False, None],
                               "TD": [short, full, False, None],
                               "TE": [short, full, False, None],
+                              "TF": [short, full, False, None],
                               })
 
         # S4: now we dirty the tree
@@ -565,6 +608,7 @@ class Repo(common.Common, unittest.TestCase):
                               "TC": [short, full, True, None],
                               "TD": ["1.0", full, False, None],
                               "TE": [short, full, True, None],
+                              "TF": [short, full, True, None],
                               })
 
         # S5: now we make one commit past the tag
@@ -577,6 +621,7 @@ class Repo(common.Common, unittest.TestCase):
                               "TC": [short, full, False, None],
                               "TD": ["0+unknown", full, False, NOTAG],
                               "TE": [short, full, False, None],
+                              "TF": [short, full, False, None],
                               })
 
         # S6: dirty the post-tag tree
@@ -589,6 +634,7 @@ class Repo(common.Common, unittest.TestCase):
                               "TC": [short, full, True, None],
                               "TD": ["0+unknown", full, False, NOTAG],
                               "TE": [short, full, True, None],
+                              "TF": [short, full, True, None],
                               })
 
 
@@ -643,6 +689,30 @@ class Repo(common.Common, unittest.TestCase):
         self.assertTrue(os.path.isdir(target))
         self.check_version(target, state, "TE", exps["TE"])
 
+        # TF: unpacked python -m build --sdist tarball
+        pyproject_path = Path(self.projdir) / "pyproject.toml"
+        if not pyproject_path.exists():
+            return
+        dist_path = Path(self.projdir) / "dist"
+        if dist_path.exists():
+            self.rmtree(dist_path)
+        # --no-isolation makes this less finicky on slow/unreliable network
+        # connections at the cost of being potentially less "normal".
+        # We'll rely on Tox and CI to keep this environment isolated and
+        # fresh.
+        self.python("-m", "build", "--sdist", "--no-isolation")
+        files = os.listdir(dist_path)
+        self.assertTrue(len(files)==1, files)
+        distfile = files[0]
+        self.assertEqual(distfile, "demo-%s.tar.gz" % exps["TF"][0])
+        fn = os.path.join(dist_path, distfile)
+        os.mkdir(self.subpath("out/TF"))
+        with tarfile.open(fn) as t:
+            t.extractall(path=self.subpath("out/TF"))
+        target = self.subpath("out/TF/demo-%s" % exps["TF"][0])
+        self.assertTrue(os.path.isdir(target))
+        self.check_version(target, state, "TF", exps["TF"])
+
     def check_version(self, workdir, state, tree, exps):
         exp_version, exp_full, exp_dirty, exp_error = exps
         if VERBOSE: print("== starting %s %s" % (state, tree))
